// src/textMateRegistry.ts
import { Registry } from "vscode-textmate";
import {
  createOnigScanner,
  createOnigString,
  loadWASM
} from "vscode-oniguruma";
async function initializeTextMateRegistry() {
  const wasmModule = await import("./onig-ZD7LO4CH.mjs");
  const response = await fetch(wasmModule.default);
  const arrayBuffer = await response.arrayBuffer();
  await loadWASM(arrayBuffer);
  return new Registry({
    onigLib: Promise.resolve({
      createOnigScanner,
      createOnigString
    }),
    async loadGrammar() {
      return null;
    }
  });
}

// src/useTextMateLanguages.tsx
import {
  useEffect as useEffect2,
  useState as useState2
} from "react";

// src/parser/index.ts
import {
  INITIAL
} from "vscode-textmate";
import {
  StreamLanguage
} from "@codemirror/language";

// src/parser/scopeMapper.ts
import { LRUCache } from "vscode-languageserver-protocol";
var _ScopeMapper = class {
  constructor(scopeMap) {
    this.scopeMap = scopeMap;
    this.cache = new LRUCache(2e3);
  }
  firstMatchingToken(scopes) {
    let i = scopes.length - 1;
    let cmToken = null;
    do {
      cmToken = this.toToken(scopes[i--]);
    } while (!cmToken && i >= 0);
    return cmToken;
  }
  toToken(scope) {
    if (!this.cache.has(scope)) {
      const subtree = this.walk(scope.split(_ScopeMapper.dotRE));
      if (subtree) {
        this.cache.set(scope, subtree);
      }
    }
    return this.cache.get(scope);
  }
  walk(scopeSegments, mapping = this.scopeMap) {
    const first = scopeSegments.shift();
    if (first === void 0)
      return null;
    const node = mapping[first];
    if (node && typeof node === "object") {
      return this.walk(scopeSegments, node) || node.$ || null;
    }
    return null;
  }
};
var ScopeMapper = _ScopeMapper;
ScopeMapper.dotRE = /\./;

// src/parser/defaultCMScopeMap.ts
var defaultCMScopeMap = {
  comment: { $: "comment" /* Comment */ },
  constant: {
    $: "def" /* Def */,
    character: { escape: { $: "string-2" /* String2 */ } },
    language: { $: "atom" /* Atom */ },
    numeric: { $: "number" /* Number */ },
    other: {
      email: { link: { $: "link" /* Link */ } },
      symbol: {
        $: "def" /* Def */
      }
    }
  },
  entity: {
    name: {
      class: { $: "def" /* Def */ },
      function: { $: "def" /* Def */ },
      tag: { $: "tag" /* Tag */ },
      type: {
        $: "variable-2" /* Variable2 */,
        class: { $: "variable" /* Variable */ }
      }
    },
    other: {
      "attribute-name": { $: "attribute" /* Attribute */ },
      "inherited-class": {
        $: "def" /* Def */
      }
    },
    support: {
      function: {
        $: "def" /* Def */
      }
    }
  },
  keyword: {
    $: "keyword" /* Keyword */,
    operator: { $: "operator" /* Operator */ },
    other: { "special-method": "def" /* Def */ }
  },
  punctuation: {
    $: "operator" /* Operator */,
    definition: {
      comment: { $: "comment" /* Comment */ },
      tag: { $: "bracket" /* Bracket */ }
    }
  },
  storage: { $: "keyword" /* Keyword */ },
  string: {
    $: "string" /* String */,
    regexp: { $: "string-2" /* String2 */ }
  },
  support: {
    class: { $: "def" /* Def */ },
    constant: { $: "variable-2" /* Variable2 */ },
    function: { $: "def" /* Def */ },
    type: { $: "type" /* Type */ },
    variable: {
      $: "variable-2" /* Variable2 */,
      property: { $: "property" /* Property */ }
    }
  },
  variable: {
    $: "def" /* Def */,
    language: {
      $: "variable-2" /* Variable3 */
    },
    other: {
      object: {
        $: "variable" /* Variable */,
        property: { $: "property" /* Property */ }
      },
      property: { $: "property" /* Property */ }
    },
    parameter: { $: "def" /* Def */ }
  }
};

// src/parser/index.ts
function createTokenizer(grammar, scopeMap) {
  const tokenMapper = new ScopeMapper(scopeMap);
  return (stream, state) => {
    const {
      pos,
      string: str
    } = stream;
    if (pos === 0) {
      const {
        ruleStack,
        tokens
      } = grammar.tokenizeLine(str, state.ruleStack);
      state.tokensCache = tokens.slice();
      state.ruleStack = ruleStack;
    }
    const { tokensCache } = state;
    const nextToken = tokensCache.shift();
    if (!nextToken) {
      stream.skipToEnd();
      return "";
    }
    const {
      endIndex,
      scopes
    } = nextToken;
    stream.eatWhile(() => stream.pos < endIndex);
    return tokenMapper.firstMatchingToken(scopes) || "";
  };
}
function createStreamParser(scopeName, grammar, scopeMap = defaultCMScopeMap) {
  const tokenizer = createTokenizer(grammar, scopeMap);
  return {
    name: scopeName,
    copyState: (state) => ({
      ruleStack: state.ruleStack.clone(),
      tokensCache: [...state.tokensCache]
    }),
    startState: () => ({
      ruleStack: INITIAL,
      tokensCache: []
    }),
    token: tokenizer
  };
}
function createLanguage(scopeName, grammar) {
  const streamParser = createStreamParser(scopeName, grammar);
  return StreamLanguage.define(streamParser);
}

// src/useTextMateRegistry.ts
import {
  useEffect,
  useState
} from "react";
function useTextMateRegistry() {
  const [registry, setRegistry] = useState();
  useEffect(function initialize() {
    const reg = initializeTextMateRegistry();
    setRegistry(reg);
    return () => {
      reg.then((r) => r.dispose());
      setRegistry((r) => r === reg ? void 0 : r);
    };
  }, []);
  return registry;
}
var useTextMateRegistry_default = useTextMateRegistry;

// src/useTextMateLanguages.tsx
function useTextMateLanguages({ textMateGrammars }) {
  const [languages, setLanguages] = useState2({ languages: {} });
  const registry = useTextMateRegistry_default();
  useEffect2(function createLanguageParsers() {
    if (!registry)
      return;
    setLanguages({
      id: registry,
      languages: {}
    });
    textMateGrammars.forEach(async (textMate) => {
      const reg = await registry;
      const grammar = await reg.addGrammar(textMate);
      const language = createLanguage(textMate.scopeName, grammar);
      setLanguages((l) => l.id === registry ? {
        ...l,
        languages: {
          ...l.languages,
          [textMate.scopeName]: language
        }
      } : l);
    });
    return () => {
      setLanguages((l) => l.id === registry ? { languages: {} } : l);
    };
  }, [registry, textMateGrammars]);
  return languages.languages;
}
export {
  createLanguage,
  initializeTextMateRegistry,
  useTextMateLanguages
};
//# sourceMappingURL=index.mjs.map